import os
import sys
import shutil
import logging
import argparse
from pathlib import Path
from typing import List, Iterable

from pycapo import CapoConfig
from sqlalchemy import create_engine
from sqlalchemy.exc import ArgumentError
from sqlalchemy.orm import sessionmaker, Session
from workspaces.schema import WorkflowRequest

_DESCRIPTION = """
Temporary directory eraser. Removes directories created by executing workflows that are no longer in use.
Defaults to checking $(HOME)/workspaces_tmp for temporary dirs.
"""

logger = logging.getLogger("tmpdir_eraser")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(stream=sys.stdout))


def get_database_url() -> str:
    """
    Get URL for Workspaces database

    :return: Database URL
    """
    capo = CapoConfig().settings("metadataDatabase")
    url = capo.jdbcUrl.replace("jdbc:", "").replace(
        "://", f"://{capo.jdbcUsername}:{capo.jdbcPassword}@"
    )
    return url


def connect_to_database(db_url: str = get_database_url()) -> Session:
    """
    Connect to Workspaces database

    :param db_url: URL of the database
    :return: Database session
    """
    try:
        engine = create_engine(db_url)
    except ArgumentError as e:
        logger.error(f"ERROR: Invalid database URL {db_url}.")
        sys.exit(1)

    session = sessionmaker(bind=engine)
    return session()


def query_for_tmpdirs(session: Session) -> List[str]:
    """
    Query the database for tmpdirs that are still in use by their WorkflowRequests

    :param session: Database session
    :return: List of dirs still in use
    """
    dirs = session.query(WorkflowRequest.results_dir).all()
    session.close()
    return dirs


def list_difference(l1: Iterable, l2: Iterable) -> Iterable:
    """
    Return the contents of l1 with the contents of l2 removed from it

    :param l1: List to subtract from
    :param l2: List of items to subtract from l1
    :return: Subtracted list
    """
    return [item for item in l1 if item not in l2]


def get_unused_dirs(
    path_to_tmpdirs: str = str(Path.home()) + "/workspaces_tmp",
) -> List[str]:
    """
    Get dirs that are not used by any WorkflowRequests

    :param path_to_tmpdirs: Path to temporary directories
    :return: List of unused dirs
    """
    all_dirs = os.listdir(path_to_tmpdirs)
    used_dirs = [
        d[0].replace(path_to_tmpdirs + "/", "")
        for d in query_for_tmpdirs(connect_to_database())
    ]
    # Remove used dirs from list of all dirs
    unused_dirs = list_difference(all_dirs, used_dirs)
    return [f"{path_to_tmpdirs}/{d}" for d in unused_dirs]


def erase_dirs(dirs_to_erase: List[str]):
    """
    Erase given directories, operating-system-willing

    :param dirs_to_erase: Directories to be erased
    :raises OSError: If dir is not found or otherwise not able to be deleted
    """
    for d in dirs_to_erase:
        try:
            shutil.rmtree(d, ignore_errors=False, onerror=None)
        except OSError as e:
            # Print error but continue trying to erase
            logger.error(e)


def make_arg_parser():
    parser = argparse.ArgumentParser(
        "tmpdir_eraser",
        description=_DESCRIPTION,
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.set_defaults(func=get_unused_dirs)
    parser.add_argument(
        "-p",
        "--path-to-dirs",
        nargs=1,
        type=str,
        required=False,
        help="path to Workspaces temp directories",
    )
    return parser


def main():
    args = make_arg_parser().parse_args()
    if args.path_to_dirs:
        dirs = get_unused_dirs(args.path_to_dirs[0])
    else:
        dirs = get_unused_dirs()
    erase_dirs(dirs)


if __name__ == "__main__":
    main()
