from typing import Dict, List, Union
import json

import requests
from pycapo import CapoConfig
from workspaces.schema import (
    Workflow,
    WorkflowTemplate,
    WorkflowRequest,
    WorkflowRequestFile,
)
from workspaces.services import WorkflowService
from workspaces.workflow_interfaces import (
    WorkflowInfoIF,
    WorkflowIF,
    WorkflowRequestIF,
    WorkflowRequestFileIF,
)


class FakeWorkflowInfo(WorkflowInfoIF):
    """
    A fake workflow info. Providing this as the substrate to other workflow
    services for testing will enable us to ensure that the right things are
    happening under the hood.
    """

    def __init__(self):
        self.requests = []

    def lookup_workflow_request(self, request_id: int) -> WorkflowIF:
        return self.requests[request_id - 1]

    def all_workflows(self) -> List[WorkflowIF]:
        return [self.lookup_workflow_definition("null")]

    def create_workflow_request(
        self, workflow: Union[str, WorkflowIF], argument: Dict
    ) -> WorkflowRequestIF:
        workflow_name = (
            workflow.workflow_name if hasattr(workflow, "workflow_name") else workflow
        )
        request = WorkflowRequest(workflow_name=workflow_name, argument=argument)
        self.save_request(request)
        return request

    def save_request(self, request: WorkflowRequestIF):
        request.workflow_request_id = len(self.requests) + 1
        self.requests.append(request)

    def save_file(
        self, request: WorkflowRequestIF, filename: str, content: bytes
    ) -> WorkflowRequestFileIF:
        request.files.append(WorkflowRequestFile(filename=filename, content=content))

    def lookup_workflow_definition(self, name: str) -> "Workflow":
        if name == "null":
            w = Workflow(workflow_name="null")
            w.templates.append(
                WorkflowTemplate(
                    filename="null.condor",
                    content=b'executable = null\narguments = "-n"\nlog = condor.log\n\nqueue',
                )
            )
            return w
        raise NotImplementedError


info = FakeWorkflowInfo()


def test_workflow_creation():
    # This seems like a bit of a blunt instrument for testing the service. We should probably break the service down
    # into smaller pieces.
    global info
    service = WorkflowService(info)

    # create request
    request = info.create_workflow_request(
        info.lookup_workflow_definition("null"), json.loads('{"argument":"-n"}')
    )

    # did the workflow info receive a new workflow request?
    assert request in info.requests

    # execute the null workflow
    # service.execute(request, [])


def test_workflow_request_execution():
    """
    Test whether a null workflow request can properly be executed
    """
    settings = CapoConfig().settings("edu.nrao.archive.workspaces.WorkflowSettings")
    r1 = requests.post(f"{settings.serviceUrl}/workflows/null/requests/create?args=-g")
    request_id = r1.json()['workflow_request_id']
    r2 = requests.post(f"{settings.serviceUrl}/workflows/requests/{request_id}/submit")
    # Assert workflow request execution was received with no error
    assert r2.status_code == 200
    r3 = requests.get(f"{settings.serviceUrl}/workflows/requests/{request_id}")
    # Assert results_dir was properly assigned
    assert len(r3.json()["results_dir"]) > 0
    # Assert workflow request is in completed state
    assert r3.json()["state"] == "Complete"
